@@ -1,23 +1,21 @@
 import logging
 import os
 
-from dvc.path_info import PathInfo
 from dvc.scheme import Schemes
 from dvc.system import System
-from dvc.utils import is_exec, tmp_fname
+from dvc.utils import tmp_fname
 from dvc.utils.fs import copy_fobj_to_file, copyfile, makedirs, move, remove
 
-from ..progress import DEFAULT_CALLBACK
-from .base import BaseFileSystem
+from ._callback import DEFAULT_CALLBACK
+from .base import FileSystem
 
 logger = logging.getLogger(__name__)
 
 
-class LocalFileSystem(BaseFileSystem):
+class LocalFileSystem(FileSystem):
     sep = os.sep
 
     scheme = Schemes.LOCAL
-    PATH_CLS = PathInfo
     PARAM_CHECKSUM = "md5"
     PARAM_PATH = "path"
     TRAVERSE_PREFIX_LEN = 2
@@ -29,28 +27,29 @@ class LocalFileSystem(BaseFileSystem):
         self.fs = LocalFS()
 
     @staticmethod
-    def open(path_info, mode="r", encoding=None, **kwargs):
-        return open(path_info, mode=mode, encoding=encoding)
+    def open(path, mode="r", encoding=None, **kwargs):
+        return open(path, mode=mode, encoding=encoding)
 
-    def exists(self, path_info) -> bool:
-        assert isinstance(path_info, str) or path_info.scheme == "local"
+    def exists(self, path) -> bool:
         # TODO: replace this with os.path.exists once the problem is fixed on
         # the fsspec https://github.com/intake/filesystem_spec/issues/742
-        return os.path.lexists(path_info)
+        return os.path.lexists(path)
 
-    def checksum(self, path_info) -> str:
-        return self.fs.checksum(path_info)
+    def checksum(self, path) -> str:
+        from fsspec.utils import tokenize
 
-    def isfile(self, path_info) -> bool:
-        return os.path.isfile(path_info)
+        st = os.stat(path)
 
-    def isdir(self, path_info) -> bool:
-        return os.path.isdir(path_info)
+        return str(int(tokenize([st.st_ino, st.st_mtime, st.st_size]), 16))
 
-    def iscopy(self, path_info):
-        return not (
-            System.is_symlink(path_info) or System.is_hardlink(path_info)
-        )
+    def isfile(self, path) -> bool:
+        return os.path.isfile(path)
+
+    def isdir(self, path) -> bool:
+        return os.path.isdir(path)
+
+    def iscopy(self, path):
+        return not (System.is_symlink(path) or System.is_hardlink(path))
 
     def walk(self, top, topdown=True, onerror=None, **kwargs):
         """Directory fs generator.
@@ -63,45 +62,33 @@ class LocalFileSystem(BaseFileSystem):
         ):
             yield os.path.normpath(root), dirs, files
 
-    def walk_files(self, path_info, **kwargs):
-        for root, _, files in self.walk(path_info):
+    def find(self, path, prefix=None):
+        for root, _, files in self.walk(path):
             for file in files:
                 # NOTE: os.path.join is ~5.5 times slower
-                yield PathInfo(f"{root}{os.sep}{file}")
+                yield f"{root}{os.sep}{file}"
 
-    def is_empty(self, path_info):
-        if self.isfile(path_info) and os.path.getsize(path_info) == 0:
+    def is_empty(self, path):
+        if self.isfile(path) and os.path.getsize(path) == 0:
             return True
 
-        if self.isdir(path_info) and len(os.listdir(path_info)) == 0:
+        if self.isdir(path) and len(os.listdir(path)) == 0:
             return True
 
         return False
 
-    def remove(self, path_info):
-        if isinstance(path_info, PathInfo):
-            if path_info.scheme != "local":
-                raise NotImplementedError
-        remove(path_info)
+    def remove(self, path):
+        remove(path)
 
-    def makedirs(self, path_info, **kwargs):
-        makedirs(path_info, exist_ok=kwargs.pop("exist_ok", True))
-
-    def isexec(self, path_info):
-        mode = self.info(path_info)["mode"]
-        return is_exec(mode)
+    def makedirs(self, path, **kwargs):
+        makedirs(path, exist_ok=kwargs.pop("exist_ok", True))
 
     def move(self, from_info, to_info):
-        if (
-            isinstance(from_info, PathInfo) and from_info.scheme != "local"
-        ) or (isinstance(to_info, PathInfo) and to_info.scheme != "local"):
-            raise NotImplementedError
-
-        self.makedirs(to_info.parent)
+        self.makedirs(self.path.parent(to_info))
         move(from_info, to_info)
 
     def copy(self, from_info, to_info):
-        tmp_info = to_info.parent / tmp_fname("")
+        tmp_info = self.path.join(self.path.parent(to_info), tmp_fname(""))
         try:
             copyfile(from_info, tmp_info)
             os.rename(tmp_info, to_info)
@@ -110,8 +97,8 @@ class LocalFileSystem(BaseFileSystem):
             raise
 
     def upload_fobj(self, fobj, to_info, **kwargs):
-        self.makedirs(to_info.parent)
-        tmp_info = to_info.parent / tmp_fname("")
+        self.makedirs(self.path.parent(to_info))
+        tmp_info = self.path.join(self.path.parent(to_info), tmp_fname(""))
         try:
             copy_fobj_to_file(fobj, tmp_info)
             os.rename(tmp_info, to_info)
@@ -124,8 +111,8 @@ class LocalFileSystem(BaseFileSystem):
         System.symlink(from_info, to_info)
 
     @staticmethod
-    def is_symlink(path_info):
-        return System.is_symlink(path_info)
+    def is_symlink(path):
+        return System.is_symlink(path)
 
     def hardlink(self, from_info, to_info):
         # If there are a lot of empty files (which happens a lot in datasets),
@@ -150,20 +137,21 @@ class LocalFileSystem(BaseFileSystem):
         System.hardlink(from_info, to_info)
 
     @staticmethod
-    def is_hardlink(path_info):
-        return System.is_hardlink(path_info)
+    def is_hardlink(path):
+        return System.is_hardlink(path)
 
     def reflink(self, from_info, to_info):
         System.reflink(from_info, to_info)
 
-    def info(self, path_info):
-        return self.fs.info(path_info)
+    def info(self, path):
+        return self.fs.info(path)
 
     def put_file(
         self, from_file, to_info, callback=DEFAULT_CALLBACK, **kwargs
     ):
-        makedirs(to_info.parent, exist_ok=True)
-        tmp_file = tmp_fname(to_info)
+        parent = self.path.parent(to_info)
+        makedirs(parent, exist_ok=True)
+        tmp_file = self.path.join(parent, tmp_fname())
         copyfile(from_file, tmp_file, callback=callback)
         os.replace(tmp_file, to_info)
 
@@ -171,3 +159,6 @@ class LocalFileSystem(BaseFileSystem):
         self, from_info, to_file, callback=DEFAULT_CALLBACK, **kwargs
     ):
         copyfile(from_info, to_file, callback=callback)
+
+
+localfs = LocalFileSystem()
